local assets=
{
	Asset("ANIM", "anim/armor_skeleton.zip"),
	
	Asset("ATLAS", "images/inventoryimages/nightmareribs.xml"),
    Asset("IMAGE", "images/inventoryimages/nightmareribs.tex"),
}

local function OnBlocked(owner) 
    owner.SoundEmitter:PlaySound("dontstarve/wilson/hit_armour") 
end

local function onequip(inst, owner)
	inst.components.armor.equipped = true
    owner.AnimState:OverrideSymbol("swap_body", "armor_skeleton", "swap_body")
    inst:ListenForEvent("blocked", OnBlocked, owner)
end

local function onunequip(inst, owner) 
	inst.components.armor.equipped = false
    owner.AnimState:ClearOverrideSymbol("swap_body")
    inst:RemoveEventCallback("blocked", OnBlocked, owner)
end

local function fn(Sim)
	local inst = CreateEntity()
    
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("armor_skeleton")
    inst.AnimState:SetBuild("armor_skeleton")
    inst.AnimState:PlayAnimation("anim")
    
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.imagename = "nightmareribs"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/nightmareribs.xml"

    inst:AddComponent("armor")
	local armorComp = inst.components.armor
    armorComp:InitCondition(1, 0)
	armorComp:SetName("nightmareribs")
	inst.tag = "nightmareribs"
	inst.activated = false
	-- Constantly lose charge until 0%
	inst:DoPeriodicTask(0.25, 
		function() 
			if armorComp:GetPercent() >= 1 then
				inst.activated = true
				armorComp:SetPercent(0)
				inst:AddTag("forcefield")
				inst.components.armor:SetAbsorption(TUNING.FULL_ABSORPTION)
				local fx = SpawnPrefab("forcefieldfx")
				fx.entity:SetParent(GetPlayer().entity)
				fx.Transform:SetPosition(0, 0.2, 0)
				local fx_hitanim = function()
					fx.AnimState:PlayAnimation("hit")
					fx.AnimState:PushAnimation("idle_loop")
				end
				fx:ListenForEvent("blocked", fx_hitanim, GetPlayer())

				inst.components.armor.ontakedamage = function(inst, damage_amount)
					if GetPlayer() then
						local sanity = GetPlayer().components.sanity
						if sanity then
							local unsaneness = damage_amount * TUNING.ARMOR_RUINSHAT_DMG_AS_SANITY
							sanity:DoDelta(-unsaneness, false)
						end
					end
				end

				inst.active = true
				GetPlayer():DoTaskInTime(--[[Duration]] TUNING.ARMOR_RUINSHAT_DURATION * 1.5, function()
					inst.activated = false
					fx:RemoveEventCallback("blocked", fx_hitanim, GetPlayer())
					fx.kill_fx(fx)
					if inst:IsValid() then
						inst:RemoveTag("forcefield")
						inst.components.armor.ontakedamage = nil
						inst.components.armor:SetAbsorption(TUNING.ARMOR_RUINSHAT_ABSORPTION)
						GetPlayer():DoTaskInTime(--[[Cooldown]] TUNING.ARMOR_RUINSHAT_COOLDOWN, function() inst.active = false end)
					end
				end)
			end
			if armorComp:GetPercent() > 0 then
				GetPlayer().components.health:SetAbsorptionAmount(armorComp:GetPercent() - (0.0025 + 0.035 * armorComp:GetPercent()^2))
				armorComp:SetPercent(armorComp:GetPercent() - (0.0025 + 0.035 * armorComp:GetPercent()^2))
			end
			if armorComp:GetPercent() <= 0 then
				GetPlayer().components.health:SetAbsorptionAmount(0)
				armorComp:SetPercent(0)
			end
			if not armorComp.equipped then
				GetPlayer().components.health:SetAbsorptionAmount(0)
			end
		end
	)
    
    inst:AddComponent("equippable")
    inst.components.equippable.equipslot = EQUIPSLOTS.BODY
    
    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip )
    
    return inst
end

return Prefab( "common/inventory/nightmareribs", fn, assets) 
