local MakePlayerCharacter = require "prefabs/player_common"

local assets = {

        Asset( "ANIM", "anim/player_basic.zip" ),
        Asset( "ANIM", "anim/player_idles_shiver.zip" ),
        Asset( "ANIM", "anim/player_actions.zip" ),
        Asset( "ANIM", "anim/player_actions_axe.zip" ),
        Asset( "ANIM", "anim/player_actions_pickaxe.zip" ),
        Asset( "ANIM", "anim/player_actions_shovel.zip" ),
        Asset( "ANIM", "anim/player_actions_blowdart.zip" ),
        Asset( "ANIM", "anim/player_actions_eat.zip" ),
        Asset( "ANIM", "anim/player_actions_item.zip" ),
        Asset( "ANIM", "anim/player_actions_uniqueitem.zip" ),
        Asset( "ANIM", "anim/player_actions_bugnet.zip" ),
        Asset( "ANIM", "anim/player_actions_fishing.zip" ),
        Asset( "ANIM", "anim/player_actions_boomerang.zip" ),
        Asset( "ANIM", "anim/player_bush_hat.zip" ),
        Asset( "ANIM", "anim/player_attacks.zip" ),
        Asset( "ANIM", "anim/player_idles.zip" ),
        Asset( "ANIM", "anim/player_rebirth.zip" ),
        Asset( "ANIM", "anim/player_jump.zip" ),
        Asset( "ANIM", "anim/player_amulet_resurrect.zip" ),
        Asset( "ANIM", "anim/player_teleport.zip" ),
        Asset( "ANIM", "anim/wilson_fx.zip" ),
        Asset( "ANIM", "anim/player_one_man_band.zip" ),
        Asset( "ANIM", "anim/shadow_hands.zip" ),
        Asset( "SOUND", "sound/sfx.fsb" ),
        Asset( "SOUND", "sound/wilson.fsb" ),
        Asset( "ANIM", "anim/beard.zip" ),
        Asset( "ANIM", "anim/warmond.zip" ),
		Asset( "ANIM", "anim/bonerang.zip"),
		Asset( "ANIM", "anim/swap_bonerang.zip"),
		Asset( "ATLAS", "images/inventoryimages/bonerang.xml" ),
		Asset( "IMAGE", "images/inventoryimages/bonerang.tex" ),
		Asset( "ATLAS", "images/inventoryimages/bonecane.xml" ),
		Asset( "IMAGE", "images/inventoryimages/bonecane.tex" ),
}
local prefabs = {"bonerang"}
-- Starting items
local start_inv = {"boneshard", "boneshard", "nightmarefuel"}

function GetSpecialCharacterString(character)
    character = string.lower(character)
    if character == "warmond" then

		local sayings =
		{
			"Ehhhhhhhhhhhhhh.",
			"",
			"*Rattles*",
			"Doot doot!",
			"Hissss",
			"Kreeaaaaaaak",
			"...",
		}

		return sayings[math.random(#sayings)]
	elseif character == "wes" then
		return ""
    end
end	

-- Doesn't increase upgrade level, just checks it
local function verifyupgrades(inst)
	-- Increase hunger decay rate based on how many times you eat nightmare fuel 5% increase per level, 100% max
	inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE * (1 + (inst.level/20)))
	-- Reduce max sanity by one point per level. You're eating nightmare fuel, what did you expect?
	inst.components.sanity.max = 50 - inst.level
	-- Reduce max health because you're going to have an easier time kiting with this
	inst.components.health.maxhealth = 50 - inst.level
	-- Speed increase!
	inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * (0.85 + (inst.level/50))
	inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * (0.85 + (inst.level/50))
end

local function applyupgrades(inst)

	if inst.level > 20 then
		inst.level = 20
	end
	--a speed upgrade is given each time nightmare fuel is consumed
	inst.level = inst.level + 1
	inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
	-- Increase hunger decay rate based on how many times you eat nightmare fuel 5% increase per level, 100% max
	inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE * (1 + (inst.level/20)))
	-- Reduce max sanity by one point per level. You're eating nightmare fuel, what did you expect?
	inst.components.sanity.max = 50 - inst.level
	-- Reduce max health because you're going to have an easier time kiting with this
	inst.components.health.maxhealth = 50 - inst.level
	-- Speed increase!
	inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * (0.85 + (inst.level/50))
	inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * (0.85 + (inst.level/50))
end

local function oneat(inst, food)
	
	if food and food.components.edible and food.components.edible.foodtype == "NIGHTMAREFUEL" then
		applyupgrades(inst)
	end
end


local function onload(inst, data)
	print("LOAD")
	if data then
		if data.level then
			inst.level = data.level
			verifyupgrades(inst)
		end
	end

end

local function onsave(inst, data)
	data.level = inst.level
end


local fn = function(inst)
	-- levels for speed upgrades
	inst.level = 0
	-- Allow Wilton to eat nightmare fuel to upgrade
	table.insert(inst.components.eater.foodprefs, "NIGHTMAREFUEL")
	inst.components.eater:SetOnEatFn(oneat)
	-- Character sounds
	inst.soundsname = "wilton"
	-- Minimap icon
	inst.MiniMapEntity:SetIcon( "wilton.png" )
	inst:AddComponent("lootdropper")
	inst:AddTag("iswilton")
	-- Wilton is very weak, with reduced health, hunger, and sanity
	inst.components.health.maxhealth = 50
	inst.components.sanity.max = 50
	inst.components.hunger:SetMax(75)
	-- Wilton hurts 10x more from starving (Do NOT starve!)
	inst.components.hunger:SetKillRate(10)
	-- In addition to having reduced max sanity, negative sanity sources (Night, monsters, etc.) are increased by 33%
	inst.components.sanity.night_drain_mult = 1.33
	inst.components.sanity.neg_aura_mult = 1.33
	-- Wilton is hurt by temperature more
	inst.components.temperature.hurtrate = 3
	-- Wilton dries off slightly faster
	if inst.components.moisture then
		inst.components.moisture.baseDryingRate = 0.1
		-- In RoG or SW, we need this extra table in order to eat nightmare fuel.
		table.insert(inst.components.eater.ablefoods, "NIGHTMAREFUEL")
	end
	-- Spawn with starter items
	--inst.components.inventory:GuaranteeItems(start_inv)
	-- 15% reduced starting speed
	-- Eventually this is cancelled out by the speed increase from nightmare fuel, but at a major cost. Sacrifices need to be made!
	inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * 0.85
	inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * 0.85
	-- Wilton is a skeleton. Who loves bones? Hounds do! Guess what that means?
	inst:AddComponent("hounded")
	-- Stronger hound waves! Yayyyyy
	inst.components.hounded:SpawnModeHeavy()
	-- Allow Wilton to craft bone-related items!
	local bonetab = {str = "Bones", sort=999, icon = "tab_arcane.tex"}
	inst.components.builder:AddRecipeTab(bonetab)
	-- Custom recipes
	local boneshard_recipe = Recipe("boneshard", {Ingredient("houndstooth", 5)}, bonetab, {SCIENCE = 0, MAGIC = 0, ANCIENT = 0})
	local bonerang_recipe = Recipe("bonerang", {Ingredient("boneshard", 2)}, bonetab, {SCIENCE = 0, MAGIC = 0, ANCIENT = 0})
	local icebonerang_recipe = Recipe("icebonerang", {Ingredient("boneshard", 5), Ingredient("bluegem", 2), Ingredient("nightmarefuel", 10)}, bonetab, {SCIENCE = 0, MAGIC = 0, ANCIENT = 0})
	local nightmarebonerang_recipe = Recipe("nightmarebonerang", {Ingredient("boneshard", 20), Ingredient("purplegem", 2), Ingredient("nightmarefuel", 40)}, bonetab, {SCIENCE = 0, MAGIC = 3, ANCIENT = 0})
	local bonecane_recipe = Recipe("bonecane", {Ingredient("boneshard", 2), Ingredient("nightmarefuel", 1)}, bonetab, {SCIENCE = 0, MAGIC = 0, ANCIENT = 0})
	local bonebat_recipe = Recipe("bonebat", {Ingredient("boneshard", 5), Ingredient("houndstooth", 5), Ingredient("nightmarefuel", 5)}, bonetab, {SCIENCE = 0, MAGIC = 0, ANCIENT = 0})
	local nightmarebonehelm_recipe = Recipe("nightmarebonehelm", {Ingredient("boneshard", 10), Ingredient("purplegem", 1), Ingredient("nightmarefuel", 15)}, bonetab, {SCIENCE = 0, MAGIC = 3, ANCIENT = 0})
	local nightmareribs_recipe = Recipe("nightmareribs", {Ingredient("boneshard", 20), Ingredient("purplegem", 2), Ingredient("nightmarefuel", 20)}, bonetab, {SCIENCE = 0, MAGIC = 3, ANCIENT = 0})
	local diviningbone_recipe = Recipe("diviningbone", {Ingredient("diviningrod", 1), Ingredient("boneshard", 5), Ingredient("nightmarefuel", 5)}, bonetab, {SCIENCE = 0, MAGIC = 0, ANCIENT = 0})
	-- If you don't resolvefilepath then the image doesn't show correctly on the builder (the crafting gui) (sometimes, not for bone shards for some reason)
	bonerang_recipe.atlas = resolvefilepath("images/inventoryimages/bonerang.xml")
	icebonerang_recipe.atlas = resolvefilepath("images/inventoryimages/icebonerang.xml")
	nightmarebonerang_recipe.atlas = resolvefilepath("images/inventoryimages/nightmarebonerang.xml")
	bonecane_recipe.atlas = resolvefilepath("images/inventoryimages/bonecane.xml")
	bonebat_recipe.atlas = resolvefilepath("images/inventoryimages/bonebat.xml")
	nightmarebonehelm_recipe.atlas = resolvefilepath("images/inventoryimages/nightmarebonehelm.xml")
	nightmareribs_recipe.atlas = resolvefilepath("images/inventoryimages/nightmareribs.xml")
	diviningbone_recipe.atlas = resolvefilepath("images/inventoryimages/diviningbone.xml")
	-- Recipe descriptions
	STRINGS.RECIPE_DESC.BONESHARD = "Tooth, bone, same thing!"
	STRINGS.RECIPE_DESC.BONERANG = "Consumes sanity on use. Infinite use!"
	STRINGS.RECIPE_DESC.ICEBONERANG = "Freezes! No damage. Heavy cost."
	STRINGS.RECIPE_DESC.NIGHTMAREBONERANG = "High dmg & speed. Very dangerous!"
	STRINGS.RECIPE_DESC.BONECANE = "+10% Movespeed!"
	STRINGS.RECIPE_DESC.BONEBAT = "Low dmg. Charges on hand for 3x damage!"
	STRINGS.RECIPE_DESC.NIGHTMAREBONEHELM = "Absorbs one free hit every 10 seconds!"
	STRINGS.RECIPE_DESC.NIGHTMARERIBS = "Charges on attack. Defense = Charge!"
	STRINGS.RECIPE_DESC.DIVININGBONE = "Leads you to your faithful companion!"
	inst.OnSave = onsave
	inst.OnLoad = onload
end


STRINGS.CHARACTER_TITLES.warmond = "The Skeleton"
STRINGS.CHARACTER_NAMES.warmond = "Wilton"
STRINGS.CHARACTER_DESCRIPTIONS.warmond = " *Very brittle\n *Can eat nightmare fuel\n *Crafts bone items"
STRINGS.CHARACTER_QUOTES.warmond = "\"Doot doot!\""
STRINGS.CHARACTERS.WARMOND = {}
STRINGS.CHARACTERS.WARMOND.DESCRIBE = {}
STRINGS.CHARACTERS.WARMOND.DESCRIBE.GENERIC = "Rattle Rattle Rattle."
	
return MakePlayerCharacter("warmond", prefabs, assets, fn, start_inv)


